<?php
/**
 * Contains 3 classes for managing settings, tables, and url links.
 *  wassupOptions: displays and update plugin settings,
 *  wassupDb: manages plugin tables, 
 *  wassupURI: generates safe url links for display.
 *
 * @package	WassUp Real-time Analytics
 * @subpackage	wassup.class.php module
 * @author	Helene D. <http://helenesit.com>
 */
//no direct request for this plugin module
$wfile=preg_replace('/\\\\/','/',__FILE__); //for windows
if(!defined('ABSPATH')|| empty($GLOBALS['wp_version'])|| preg_match('#'.preg_quote(basename($wfile)).'#',$_SERVER['PHP_SELF'])|| !empty($_SERVER['SCRIPT_FILENAME'])&& realpath($wfile)===realpath($_SERVER['SCRIPT_FILENAME'])){
	if(!headers_sent()){header('Location: /?p=404page&err=wassup403');exit;
	}elseif(function_exists('wp_die')){wp_die("Bad Request: ".esc_attr(wp_kses(preg_replace('/(&#37;|&amp;#37;|%)(?:[01][0-9A-F]|7F)/i','',$_SERVER['REQUEST_URI']),array())));exit;
	}else{die("Bad Request: ".htmlspecialchars(preg_replace('/(&#37;|&amp;#37;|%)(?:[01][0-9A-F]|7F)/i','',$_SERVER['REQUEST_URI'])));exit;}
	exit;
}
unset($wfile);
if (!class_exists('wassupOptions')) {
/**
 * Class for display, update, and validation of settings form data.
 * @author: Helene Duncker <http://helenesit.com>
 */
class wassupOptions {
	/* general/detail settings */
	var $wassup_refresh = "3";	
	var $wassup_userlevel = "8";
	var $wassup_screen_res = "800";	
	var $wassup_default_type = "everything";	
	var $wassup_default_spy_type = "everything";
	var $wassup_default_limit = "10";
	var $wassup_top10;
	var $wassup_time_format = "24";
	var $wassup_time_period = "1";

	/* recording settings */
	var $wassup_active = "1";	
	var $wassup_loggedin = "1";
	var $wassup_admin = "1";
	var $wassup_spider = "1";
	var $wassup_exclude = "";
	var $wassup_exclude_host = "";	//new in v1.9: for exclusion by hostname
	var $wassup_exclude_url = "";
	var $wassup_exclude_user = "";

	/* spam and malware settings */
	var $wassup_spamcheck = "1";
	var $wassup_spam = "1";
	var $wassup_refspam = "1";
	var $wassup_attack = "1";
	var $wassup_hack = "1";	

	/* table/file management settings */
	var $wassup_table;
	var $wassup_dbengine = "";
	var $wassup_uninstall;	//for uninstall of wassup tables
	var $delete_auto = "never";
	var $delete_filter = "";
	var $wassup_optimize = "0";	//for scheduled optimization
	var $wassup_remind_mb = "100";
	var $wassup_remind_flag = "1";
	var $delayed_insert = "1";	//New in v1.9: for use of "Delayed" option in MySQL INSERT command

	/* chart/map display settings */
	var $wassup_dashboard_chart = 0;
	var $wassup_chart = "1";
	var $wassup_chart_type = "2";
	var $wassup_geoip_map = "1";
	var $wassup_googlemaps_key;

	/* widget settings */
	//New in v1.9: removed widget settings because multi-widgets store their own settings in wordpress options table

	/* temporary action settings */
	//New in v1.9: dip,mark and other temp settings moved to new user option '_wassup_settings' so that multiple users can view wassup admin panel and customize their views
	var $whash = "";
	var $wassup_alert_message = "";	//to display alerts
	var $wassup_version = "";
	var $wassup_upgraded = 0;	//New in v1.9: upgrade timestamp

	/**
	 * PHP4 constructor.
	 *
	 * New in v1.9: optional argument to set default values for new/empty class vars.
	 * @param boolean $add_defaults 
	 * @return void
	 *
	 */
	public function wassupoptions($add_defaults=false){
		if($add_defaults)$this->_initSettings();
		else $this->loadSettings();
	}
	/** loads current settings/initializes empty class vars. */
	private function _initSettings(){
		$settings=$this->getSettings(true);
		$this->loadSettings($settings);
	}
	/** loads default settings into class vars. */
	public function loadDefaults(){
		$defaults = $this->defaultSettings();
		$this->loadSettings($defaults);
	}
	/**
	 * Return array of default values or one default variable.
	 * @param string
	 * @return string|array
	 */
	public function defaultSettings($dsetting="") {
		global $wpdb,$wdebug_mode;
		$retvalue=false;
		//New in v1.9: default settings for current user 
		if($dsetting=="user_settings" || $dsetting=="wassup_user_settings"){
		$user_defaults=array(
			'detail_filter'	=>$this->wassup_default_type,
			'detail_chart'	=>$this->wassup_chart,
			'detail_limit'	=>$this->wassup_default_limit,
			'detail_time_period'=>$this->wassup_time_period,
			'spy_filter'	=>$this->wassup_default_spy_type,
			'spy_map'	=>$this->wassup_geoip_map,
			'ualert_message'=>"",
			'unonce'	=>rand(1,999999),
			'umark' 	=>"",
			'uip'   	=>"",
			'urecid'	=>0,
			'utimestamp'	=>0,
			'uwassupid'	=>"",
			'uversion'	=>WASSUPVERSION,
			);
			return $user_defaults;
		}
		//default top stats settings
		$top10_defaults = array(
			"toplimit"=>"10",
			"topsearch"=>"1",
			"topreferrer"=>"1",
			"toppostid"=>"1",
			"toprequest"=>"1",
			"topbrowser"=>"1",
			"topos"=>"1",
			"toplocale"=>"0",
			"topvisitor"=>"0",
			"topreferrer_exclude"=>"",
			"top_nofrontpage"=>"0",
			"top_nospider"=>"0",
			);
		if($dsetting=="top10" || $dsetting=="wassup_top10" || $dsetting=="top_stats"){
			return $top10_defaults;
		}
		//general default settings
		$defaults = array(
			'wassup_active'		=>"1",
			'wassup_loggedin'	=>"1",
			'wassup_admin'		=>"1",
			'wassup_spider'		=>"1",
			'wassup_attack'		=>"1",
			'wassup_hack'		=>"1",
			'wassup_spamcheck'	=>"1",
			'wassup_spam'		=>"1",
			'wassup_refspam'	=>"1",
			'wassup_exclude'	=>"",
			'wassup_exclude_host'	=>"",
			'wassup_exclude_url'	=>"",
			'wassup_exclude_user'	=>"",
			'wassup_chart'		=>"1",
			'wassup_chart_type'	=>"2",
			'delete_auto'		=>"never",
			'delete_filter'		=>"",
			'wassup_remind_mb'	=>"100", //v1.9 new default setting
			'wassup_remind_flag'	=>"1",	//v1.9 new default setting
			'wassup_refresh'	=>"3",
			'wassup_userlevel'	=>"8",
			'wassup_screen_res'	=>"800",
			'wassup_default_type'	=>"everything",
			'wassup_default_spy_type'=>"everything",
			'wassup_default_limit'	=>"10",
			'wassup_dashboard_chart'=>"0",
			'wassup_geoip_map'	=>"1",	//New in v1.9 - new default setting
			'wassup_googlemaps_key'	=>"",
			'wassup_time_format'	=>"24",
			'wassup_time_period'	=>"1",
			'wassup_alert_message'	=>"",
			'wassup_uninstall'	=>"0",
			'wassup_optimize'=>"0",
			'wassup_top10'	=>$top10_defaults,
			'whash' 	=>"",
			'wassup_table'	=>$wpdb->prefix . "wassup",
			'wassup_dbengine'=>"",
			'delayed_insert'=>"1",
			'wassup_version'=>"",
			'wassup_upgraded'=>0,
		);
		//New in v1.9: for multisite, use main site settings as default, except for table name and engine
		if(is_multisite() && !is_main_site() && !is_network_admin()){
			$main_site_settings=get_blog_option($GLOBALS['current_site']->blog_id,'wassup_settings',$defaults);
			if(!empty($main_site_settings) && $main_site_settings!=$defaults){
				$defaults=$main_site_settings;
				$defaults['wassup_table']=$wpdb->prefix . "wassup";
			}
		}
		//never discard google maps api key with "reset-to-default"
		if(!empty($this->wassup_googlemaps_key)) $defaults['wassup_googlemaps_key']= $this->wassup_googlemaps_key;
		//never discard wassup_version' with "reset-to-default"
		if(!empty($this->wassup_version)) $defaults['wassup_version']= $this->wassup_version;
		$wassupdb_installed=false;
		//never change 'wassup_table' with "reset-to-default" ..unless table doesn't exist
		if(!empty($this->wassup_table) && wassupDb::table_exists($this->wassup_table)){
			$defaults['wassup_table']= $this->wassup_table;
			$defaults['wassup_dbengine']= $this->wassup_dbengine;
			$defaults['wassup_optimize']= $this->wassup_optimize;
			$wassupdb_installed=true;
		}elseif(is_multisite() && !is_network_admin() && !is_main_site()){
			$network_settings=get_site_option('wassup_network_settings');
			if(!empty($network_settings['wassup_table'])){
				$defaults['wassup_table']=$network_settings['wassup_table'];
				$wassupdb_installed=true;
			}elseif(wassupDb::table_exists($defaults['wassup_table'])){
				$wassupdb_installed=true;
			}
		}elseif(wassupDb::table_exists($defaults['wassup_table'])){
			$wassupdb_installed=true;
		}
		//reset table engine with reset-to-default ..after default wassup_table is set
		if($dsetting=='wassup_dbengine' || empty($dsetting)){
			if(empty($defaults['wassup_dbengine'])){
				$tengine="";
				if ($defaults['wassup_table']== $this->wassup_table){
					$tengine=$this->wassup_dbengine;
				}elseif(!$wassupdb_installed){
					$result=$wpdb->get_results("SHOW VARIABLES LIKE 'storage_engine'",ARRAY_A);
					if(!empty($result) && !is_wp_error($result) && !empty($result[0]->Value)) $tengine=$result[0]->Value;
				}else{
					//$tengine=wassupDb::get_db_setting('engine',$defaults['wassup_table']); //TODO: find cause of mysql timeout error
					$result=$wpdb->get_results("SHOW VARIABLES LIKE 'storage_engine'",ARRAY_A);
					if(!empty($result) && !is_wp_error($result) && !empty($result[0]->Value)) $tengine=$result[0]->Value;
				}
				$defaults['wassup_dbengine']=$tengine;
			}
		}
		//never change optimize schedule with reset-to-default ..unless table engine has changed
		if($dsetting=='wassup_optimize' || (empty($dbsetting) && $this->wassup_dbengine != $defaults['wassup_dbengine'])){
			$tengine=$defaults['wassup_dbengine'];
			//v1.9 bugfix: set optimization when for table engine's myisam, archive, or innodb with file-per-table option only
			if($tengine=="myisam" || $tengine=="archive"){
				$defaults['wassup_optimize']=strtotime("next Sunday 2:00am");
			}elseif($wassupdb_installed && wassupDb::is_optimizable_table($defaults['wassup_table'])){
				$defaults['wassup_optimize']=strtotime("next Sunday 2:00am");
			}else{
				$defaults['wassup_optimize']="0";
			}
		}else{
			$defaults['wassup_optimize']=$this->wassup_optimize;
		}
		//reset hash with reset-to-default
		$defaults['whash']=$this->whash; 
		if($dsetting=='whash' || empty($dsetting)){
			$defaults['whash']=$this->get_wp_hash();
		}
		//serialize top10 array for insert into wp_options
		if(empty($dsetting)) $defaults['wassup_top10']=maybe_serialize($top10_defaults);
		//check that can use delayed insert
		//$defaults['delayed_insert']=$this->delayed_insert;
		if(empty($dsetting) || $dsetting=="delayed_insert"){
			if (stristr($defaults['wassup_dbengine'],"isam")===false && stristr($defaults['wassup_dbengine'],"archive")===false){
				$defaults['delayed_insert']="0";
			}else{
				$delayed_queue_size=wassupDb::get_db_setting("delayed_queue_size");
				if(!is_numeric($delayed_queue_size) || (int)$delayed_queue_size==0){
					$defaults['delayed_insert']="0";
				}else{
					$max_delayed_threads=wassupDb::get_db_setting("max_delayed_threads");
					if((int)$max_delayed_threads==0) $defaults['delayed_insert']="0";
				}
			}
		}
		//Return default value for "dsetting" argument, if any
		if(!empty($dsetting)){
			if ($dsetting == "user_setting" || $dsetting == "wassup_user_settings"){
				$retvalue = $user_defaults;
			}elseif ($dsetting == "top10" || $dsetting == "wassup_top10" || $dsetting == "top_stats"){
				$retvalue = $top10_defaults;
			}elseif(isset($defaults[$dsetting])){
				$retvalue = $defaults[$dsetting];
			}else{
				$retvalue = false;
			}
		}else{
			$retvalue=$defaults;
		}
		return $retvalue;
	} //end defaultSettings

	/** Load up class variables with wp_option settings. */
	public function loadSettings($settings=array()){
		if(empty($settings)){
			$settings=get_option('wassup_settings');
		}
		if(!empty($settings) && is_array($settings)){
			$this->_options2class($settings);
		}else{
			return false;
		}
		return true;
	}

	/**
	 * Retrieve wassup settings from 'wp_options' in an array.
	 *  - includes optional flag argument to add new settings and omit deprecated settings.
	 * @since version 1.8
	 * @param boolean $add_defaults
	 * @return array $settings
	 */
	public function getSettings($add_defaults=false){
		global $wpdb;
		$current_opts=get_option('wassup_settings');
		if($add_defaults || empty($current_opts)){
			//in multisite, use main site as defaults
			if(is_multisite() && !is_network_admin() && !is_main_site()){
				$default_opts=get_blog_option($GLOBALS['current_site']->blog_id,'wassup_settings');
				if(empty($default_opts)) $default_opts=$this->defaultSettings();
				else $default_opts['wassup_table']=$this->defaultSettings('wassup_table');
			}else{
				$default_opts=$this->defaultSettings();
			}
			if(!empty($current_opts)){
				foreach($default_opts as $skey=>$defaultvalue){
					if(array_key_exists($skey,$current_opts))$settings[$skey]=$current_opts[$skey];
					else $settings[$skey]=$defaultvalue;
				}
			}else{
				$settings=$default_opts;
			}
		}else{
			$settings=$current_opts;
		}
		return $settings;
	} //end getSettings

	/** Save class vars as 'wassup_settings' in wp_options. */
	public function saveSettings() {
		global $wpdb;
		$settings_array = array();
		$obj = $this;
		//convert class vars into array
		foreach (array_keys(get_class_vars(get_class($obj))) as $k){
			if (is_array($obj->$k)) {
				//serialize any arrays within $obj
				if (count($obj->$k)>0) {
					$settings_array[$k] = maybe_serialize($obj->$k);
				} else {
					$settings_array[$k] = "";
				}
			} else {
				$settings_array[$k] = "{$obj->$k}";
			}
		}
		//save array to wp_options
		/* if(is_multisite() && function_exists('is_network_admin') && is_network_admin()){
			$subsite_id=$GLOBALS['current_site']->blog_id;
			update_blog_option($subsite_id,'wassup_settings', $settings_array);
		}else{
			update_option('wassup_settings', $settings_array);
		} */
		update_option('wassup_settings', $settings_array);
		return true;
	} //end saveSettings

	/** delete 'wassup_settings' from wp_options table and reset class vars to defaults. */
	public function deleteSettings(){
		global $wpdb;
		$this->loadDefaults();
		if(!is_multisite()){
			delete_option('wassup_settings');
		}else{
			if(function_exists('is_network_admin') && is_network_admin()) $subsite_id=$GLOBALS['current_site']->blog_id;
			else $subsite_id=$GLOBALS['current_blog']->blog_id;
			delete_blog_option($subsite_id,'wassup_settings');
		}
	}
	/**
	 * Reset wassup user option '_wassup_settings' to defaults
	 *  - runs when a user logs in, after upgrade/install, and with reset-to-default
	 *  - contains 2 user arguments required by 'wp_login' hook.
	 *
	 * @since v1.9
	 * @param (2) string $user_login, object(WP_User) $user
	 * @return array $wassup_user_settings
	 */
	public function resetUserSettings($user_login="",$user=false){
		global $current_user;
		if(!defined('WASSUPVERSION')) wassup_init();
		if(empty($user)) $user=$current_user;
		if(empty($user->ID)) $user=wp_get_current_user();
		$wassup_user_settings=get_user_option('_wassup_settings',$user->ID);
		if(!empty($wassup_user_settings)){
			$wassup_user_defaults=$this->defaultSettings('wassup_user_settings');
			$wassup_user_settings=$wassup_user_defaults;
			update_user_option($user->ID,'_wassup_settings',$wassup_user_settings);
		}
		return $wassup_user_settings;
	}
	/**
	 * Return an array of valid input field values or a single default value for a field in wassup settings form.
	 *  - value returned could be field value, field name, or the sql associated with the field, depending on the $meta param
	 * @param string(3) ($field,$meta,$selected)
	 * @return array
	 */
	public function getFieldOptions($field,$meta="",$selected="") {
		$field_options = array();
		$field_options_meta = array();
		$field_options_sql = array();
		$default_key = "";	//default value
		switch ($field) {
		case "wassup_screen_res":
			//"Options" setting
			$field_options = array("640","800","1024","1200",1600);
			$field_options_meta = array("&nbsp;640",
				"&nbsp;800",
				"1024",
				"1200",
				"1600");
			$default_key=1;
			break;
		case "wassup_userlevel":
			//"Options" setting
			$field_options = array("8","6","2","1","0");
			$field_options_meta = array(
				__("Administrator"),
				'&nbsp;'.__("Editor"),
				'&nbsp;'.__("Author"),
				'&nbsp;'.__("Contributor"),
				'&nbsp;'.__("Subscriber"));
			break;
		case "wassup_chart_type":
			//"Options" setting
			$field_options = array("1","2");
			$field_options_meta = array(
				__("One - 2 lines chart 1 axis","wassup"),
				__("Two - 2 lines chart 2 axes","wassup"));
			$default_key = "1";
			break;
		case "wassup_default_type":
		case "wassup_default_spy_type":
			$sitehome = wassupURI::get_sitehome();
			$wurl = parse_url(strtolower($sitehome));
			$sitehome = $wurl['host'];
			if(is_multisite() && !is_subdomain_install() && !empty($wurl['path'])) $sitehome=$wurl['host'].$wurl['path'];
			$sitedomain=rtrim(str_replace('.','\\.',$sitehome),'/ ');
			$field_options = array("everything", 
				"spider",
				"nospider",
				"spam",
				"nospam",
				"nospamspider",
				"loggedin",
				"comauthor",
				"searchengine",
				"referrer");
			$field_options_meta = array(__("Everything","wassup"),
				__("Spider","wassup"),
				__("No spider","wassup"),
				__("Spam","wassup"),
				__("No Spam","wassup"),
				__("No Spam, No Spider","wassup"),
				__("Users logged in","wassup"),
				__("Comment authors","wassup"),
				__("Referrer from search engine","wassup"),
				__("Referrer from ext link","wassup"));
			$field_options_sql = array("",
				" AND spider!=''",
				" AND spider=''",
				" AND spam>0",
				" AND spam=0",
				" AND spam=0 AND spider=''",
				" AND username!=''",
				" AND comment_author!=''",
				" AND searchengine!=''",
				" AND referrer!='' AND searchengine='' AND TRIM(LEADING 'http://' FROM TRIM(LEADING 'https://' FROM `referrer`)) NOT RLIKE '^(www".'\\.'.")?(".$sitedomain."/)'",
				//" AND referrer!='' AND referrer NOT LIKE 'http://".$sitedomain."%' AND referrer NOT LIKE 'https://".$sitedomain."%' AND referrer NOT LIKE 'http://www.".$sitedomain."%'",
				);
			break;
		case "wassup_default_limit":
			//"Options" setting, report and chart option
			$field_options = array("10","20","50","100");
			$field_options_meta = array("&nbsp;10",
				"&nbsp;20",
				"&nbsp;50",
				"100");
			break;
		case "delete_auto":
			//"Options" settings
			$field_options = array("never", 
					"-1 day",
					"-1 week", 
					"-2 weeks", 
					"-1 month", 
					"-3 months",
					"-6 months",
					"-1 year");
			$field_options_meta = array(
				__("Don't delete anything","wassup"),
				__("24 hours","wassup"),
				__("7 days","wassup"),
				__("2 weeks","wassup"),
				__("1 month","wassup"),
				__("3 months","wassup"),
				__("6 months","wassup"),
				__("1 year","wassup"));
			break;
		case "delete_filter":
			$field_options = array("all",
				"spider",
				"spam",
				"spider_spam");
			$field_options_meta = array(__("All"),
				__("Spider"),
				__("Spam"),
				__("Spider and spam","wassup"));
			$field_options_sql=array("",
				" AND `spider`!=''",
				" AND `spam`!='0' AND `spam`!=''",
				" AND (`spider`!='' OR (`spam`!='0' AND `spam`!=''))");
			break;
		case "sort_group":
			//TODO add to dislay options in Main/details screen
			$field_options = array("IP","URL");
			$field_options_meta = array(
				__("IP Address","wassup"),
				__("URL Request","wassup"));
			break;
		case "wassup_time_period": 
			//"Details" report and chart option
			$field_options = array(".05",".25","0.5","1","7","14","30","90","180","365","0");
			$field_options_meta = array(
				__("1 hour"),
				__("6 hours"),
				__("12 hours"),
				__("24 hours"),
				__("7 days"),
				__("2 weeks"),
				__("1 month"),
				__("3 months"),
				__("6 months"),
				__("1 year"),
				__("all time","wassup"),
			);
			$default_key=3; //default:meta[3]=24 hours
			break;
		default: 	//enable/disable is default
			$field_options =  array("1","0");
			$field_options_meta =  array("Enable","Disable");
		} //end switch
		if (empty($default_key)) $default_key=0;
		$retval = "";
		if ($meta == "meta") {
			//return 1 item
			if ($selected!="") {
				$key = array_search($selected,$field_options);
				if ($key) {
					$retval=$field_options_meta[$key];
				} elseif (!is_numeric($default_key)) {
					$key = array_search($default_key,$field_options);
					$retval=$field_options_meta[$key];
				} else {
					$retval=$field_options_meta[$default_key];
				}
			//return array of items
			} else {
				$retval=$field_options_meta;
			}
		} elseif ($meta == "default") {
			$retval = $default_key;
		} elseif ($meta == "sql") {
			if (!empty($field_options_sql)) {
			if ($selected!="") { //return 1 item
				$key = array_search($selected,$field_options);
				if($key) $retval=$field_options_sql[$key];
				else $retval=$field_options_sql[$default_key];
			} else { $retval=$field_options_sql; }
			}
		}else{
			$retval=$field_options;
		}
		return $retval;
	} //end getFieldOptions

	/**
	 * Generates <options> tags for use in a <select> form field.
	 * - 1st argument $itemkey must an input field name from the "wassup-options" form or from 'getFieldOptions' above.
	 * @param mixed(3) (string $itemkey,integer $selected, string $optionargs)
	 * @return string (html)
	 */
	public function showFieldOptions($itemkey,$selected="",$optionargs=""){
		$form_items =$this->getFieldOptions($itemkey);
		if(count($form_items) > 0){
			$form_items_meta=$this->getFieldOptions($itemkey,"meta");
			if($selected == ""){ 
				if(isset($this->$itemkey)){
					$selected=$this->$itemkey;
				}else{ 
					$default=$this->getFieldOptions($itemkey,"default");
					if(!empty($default) && is_numeric($default)) $selected=$form_items[$default];
					else $selected=$form_items[0];
				}
			}
			foreach($form_items as $k => $option_item){
				echo "\n\t\t".'<option value="'.$optionargs.$option_item.'"';
				if($selected==$option_item) echo ' SELECTED>';
				else echo '>';
				echo $form_items_meta[$k].'&nbsp;&nbsp;</option>';
			}
		}
	} //end showFieldOptions

	/**
	 * strip bad characters from a text or textarea input string
	 * @since v1.9
	 * @param string
	 * @return string
	 */
	public function cleanFormText($input){
		$text=wp_kses($input,array());
		return preg_replace('/([^0-9a-z\-_\.,\:*#\'" ]+)/i','',strip_tags(html_entity_decode($text)));
	}
	/** strip bad characters from a text or textarea input for URLs. @since v1.9 */
	public function cleanFormURL($input){
		$loc=wp_kses_bad_protocol($input,array('http','https','feed'));
		$loc=preg_replace('/([^0-9a-z\-_\.,\:\*#\/&\?=;% ]+)/i','',strip_tags(html_entity_decode(preg_replace('/(&#37;|&amp;#37;|%)(?:[01][0-9A-F]|7F)/i','',$loc))));
		return $loc;
	}
	/**
	 * Save settings form changes stored in $_POST global.
	 * @since v1.9
	 * @param none
	 * @return string
	 */
	public function saveFormChanges(){
		global $current_user;
		if(!is_object($current_user) || empty($current_user->ID)) wp_get_current_user();
		$admin_message=__("Nothing to do!","wassup");
		if(!empty($_POST) && current_user_can('manage_options') && !empty($_SERVER['HTTP_REFERER']) && strpos($_SERVER['HTTP_REFERER'],'page=wassup-options')!==false){
			$sitehome=wassupURI::get_sitehome();
			//New in v1.9: save multisite network settings
			$network_settings=array();
			if(is_multisite() && !empty($_POST['_network_settings'])){
				$network_settings=get_site_option('wassup_network_settings');
				$network_settings['wassup_active']=(!empty($_POST['network_active'])?"1":"0");
				if($this->network_activated_plugin()){
					$network_settings['wassup_menu']=(!empty($_POST['wassup_menu'])?"1":"0");
				}
				update_site_option('wassup_network_settings',$network_settings);
			}
			$this->wassup_active=(!empty($_POST['wassup_active'])?"1":"0");
			$this->wassup_spamcheck=(!empty($_POST['wassup_spamcheck'])?"1":"0");
			$this->wassup_screen_res=(int)$_POST['wassup_screen_res'];
			$this->wassup_userlevel=(int)$_POST['wassup_userlevel'];
			$this->wassup_dashboard_chart=(!empty($_POST['wassup_dashboard_chart'])?"1":"0");
			$this->wassup_geoip_map=(!empty($_POST['wassup_geoip_map'])?"1":"0");
			if(!empty($_POST['wassup_googlemaps_key'])){
				$not_required=__("not required","wassup");
				if(strpos($_POST['wassup_googlemaps_key'],$not_required)===false)$this->wassup_googlemaps_key=$this->cleanFormText($_POST['wassup_googlemaps_key']);
				else $this->wassup_googlemaps_key="";
			}else{
				$this->wassup_googlemaps_key="";
			}
			$this->wassup_chart=(!empty($_POST['wassup_chart'])?"1":"0");
			if(!empty($_POST['wassup_chart_type'])) $this->wassup_chart_type=(int)$_POST['wassup_chart_type'];
			else $this->wassup_chart_type="2";
			$this->wassup_time_format=$this->cleanFormText($_POST['wassup_time_format']);
			$this->wassup_time_period=$this->cleanFormText($_POST['wassup_time_period']);
			if(!empty($_POST['wassup_refresh'])){
				if(is_numeric($_POST['wassup_refresh'])&& $_POST['wassup_refresh']>0 && $_POST['wassup_refresh']<=180)
					$this->wassup_refresh=(int)$_POST['wassup_refresh'];
			}else{
				$this->wassup_refresh="0";
			}
			$this->wassup_default_type=$this->cleanFormText($_POST['wassup_default_type']);
			$this->wassup_default_limit=$this->cleanFormText($_POST['wassup_default_limit']);
			$top_ten=array(
				"toplimit"=>(isset($_POST['toplimit'])?(int)$_POST['toplimit']:"10"),
				"topsearch"=>(isset($_POST['topsearch'])?$_POST['topsearch']:"0"),
				"topreferrer"=>(!empty($_POST['topreferrer'])?"1":"0"),
				"toppostid"=>(!empty($_POST['toppostid'])?"1":"0"),
				"toprequest"=>(!empty($_POST['toprequest'])?"1":"0"),
				"topbrowser"=>(!empty($_POST['topbrowser'])?"1":"0"),
				"topos"=>(!empty($_POST['topos'])?"1":"0"),
				"toplocale"=>(!empty($_POST['toplocale'])?"1":"0"),
				"topvisitor"=>(!empty($_POST['topvisitor'])?"1":"0"),
				"topreferrer_exclude"=>$this->cleanFormURL($_POST['topreferrer_exclude'],'*'),
				"top_nofrontpage"=>(!empty($_POST['top_nofrontpage'])?"1":"0"),
				"top_nospider"=>(!empty($_POST['top_nospider'])?"1":"0"),
			);
			$this->wassup_top10=maybe_serialize($top_ten);
			$this->wassup_loggedin=(!empty($_POST['wassup_loggedin'])?"1":"0");
			$this->wassup_admin=(!empty($_POST['wassup_admin'])?"1":"0");
			$this->wassup_spider=(!empty($_POST['wassup_spider'])?"1":"0");
			$this->wassup_spam=(!empty($_POST['wassup_spam'])?"1":"0");
			$this->wassup_refspam=(!empty($_POST['wassup_refspam'])?"1":"0");
			$this->wassup_hack=(!empty($_POST['wassup_hack'])?"1":"0");
			$this->wassup_attack=(!empty($_POST['wassup_attack'])?"1":"0");
			if($_POST['wassup_exclude'] != $this->wassup_exclude)
				$this->wassup_exclude=$this->cleanFormText($_POST['wassup_exclude']);
			if ($_POST['wassup_exclude_host'] != $this->wassup_exclude_host)
				$this->wassup_exclude_host=$this->cleanFormText($_POST['wassup_exclude_host']);
			if ($_POST['wassup_exclude_user'] != $this->wassup_exclude_user)
				$this->wassup_exclude_user=$this->cleanFormText($_POST['wassup_exclude_user']);
			if ($_POST['wassup_exclude_url'] != $this->wassup_exclude_url){
				if(wassupURI::is_root_url($sitehome))
					$this->wassup_exclude_url=str_replace($sitehome,'',$this->cleanFormURL($_POST['wassup_exclude_url']));
				else
					$this->wassup_exclude_url=str_replace(rtrim($sitehome,'/'),'/',$this->cleanFormURL($_POST['wassup_exclude_url']));
			}
			if(!empty($_POST['wassup_remind_mb'])&& is_numeric($_POST['wassup_remind_mb'])){
				$this->wassup_remind_mb=(int)$_POST['wassup_remind_mb'];
			}elseif(!isset($_POST['wassup_remind_mb'])){
				$this->wassup_remind_mb=100;
			}else{
				$this->wassup_remind_mb=0;
			}
			if(!empty($_POST['wassup_remind_flag'])){
				$this->wassup_remind_flag=(int)$_POST['wassup_remind_flag'];
				if(empty($_POST['wassup_remind_mb']))$this->wassup_remind_mb=100;
			}
			if(isset($_POST['delete_auto'])){
				$this->delete_auto=$this->cleanFormText($_POST['delete_auto']);
				if(isset($_POST['delete_filter']))$this->delete_filter=$this->cleanFormText($_POST['delete_filter']);
				//schedule daily delete auto
				if(!empty($this->delete_auto) && $this->delete_auto!="never"){
					$starttime=wp_next_scheduled('wassup_scheduled_purge');
					if(empty($starttime)){
						//start purge at 2am
						$starttime=strtotime("tomorrow 2:00am");
						wp_schedule_event($starttime,'daily','wassup_scheduled_purge');
					}
				}else{
					wp_clear_scheduled_hook('wassup_scheduled_purge');
				}
			}else{
				wp_clear_scheduled_hook('wassup_scheduled_purge');
			}
			//New in v1.9: optimization and delayed_insert options
			if(isset($_POST['wassup_optimize_on'])){
				if($this->wassup_optimize=="0")
					$this->wassup_optimize=$this->defaultSettings('wassup_optimize');
			}else{
				$this->wassup_optimize="0";
			}
			$this->delayed_insert=(!empty($_POST['delayed_insert'])?"1":"0");
			if (!empty($_POST['wassup_dbengine'])) $this->wassup_dbengine=$this->cleanFormText($_POST['wassup_dbengine']);
			if($this->saveSettings()){
				$admin_message=__("Wassup options updated successfully","wassup")."." ;
				$this->resetUserSettings($current_user->user_login,$current_user);
			}
		}
		return $admin_message;
	} //end saveFormChanges

	/**
	 * get timezone (and offset) directly from the host server.
	 * @since 1.8
	 * @param none
	 * @return array (*string $timezone, *string $offset)
	 */
	static function getHostTimezone($getoffset=false) {
		$hostTimezone = __("Unknown");
		$nix_server="";
		//run *nix 'date' command to get offset from host server system
		//'date' function for timezone not supported on Windows
		if (defined('PHP_OS') && strtoupper(substr(PHP_OS, 0, 3)) !== 'WIN') {
			$nix_server = PHP_OS;
		} else {
			if (!empty($_SERVER['SERVER_SOFTWARE'])) {
				$php_os = $_SERVER['SERVER_SOFTWARE'];
			} elseif (function_exists('apache_get_version')) { 
				$php_os = apache_get_version();
			}
			if (preg_match('/(nix|bsd|os\s?x|ux|darwin|sun)/i',$php_os)>0) {
				$nix_server = $php_os;
			}
		}
		if (!empty($nix_server) && function_exists('shell_exec')) {
			//check that 'shell_exec' is enabled (no safe_mode)
			$safe_mode="off";
			if (version_compare(PHP_VERSION,'5.4','<')) {
				$safe_mode = @ini_get('safe_mode');
				if ($safe_mode!= "on" && $safe_mode!= "1")
					$safe_mode="off";
			}
			if ($safe_mode=='off' && !in_array('shell_exec',array_map('trim',explode(',', @ini_get('disable_functions'))))) {
				if ($getoffset) {
					$hostTZ = @shell_exec('date +"%Z|%z"');
					//in case shell_exec is disabled
					if (strpos($hostTZ,'|')!==false) {
						$hostTimezone = explode('|',$hostTZ);
						$hostTimezone[1] = substr($hostTimezone[1],0,3);
					}
				} else {
					$hostTZ = @shell_exec('date +"%Z"');
					//in case shell_exec is disabled
					if (!empty($hostTZ)) {
						$hostTimezone = $hostTZ;
					}
				}
			} //end if safe_mode
		}
		return $hostTimezone;
	} //end getHostTimezone

	/**
	 * Return true if site uses American English format for dates and numbers
	 * @since v1.9
	 * @param string
	 * @return boolean
	 */
	static function is_USAdate($ftype="date"){
		$is_usaformat=false;
		//check for US|Euro date format in wordpress options
		$wp_dateformat=get_option('date_format');
		$i=strpos($wp_dateformat,'j');
		if($i===false)$i=strpos($wp_dateformat,'d');
		if($i>1 && strpos($wp_dateformat,'Y')!==0)$is_usaformat=true;
		return $is_usaformat;
	}

	/**
	 * Verify that a user has administrator role.
	 *  - "username" param can be empty, string, or WPUser object
	 * @since v1.9
	 * @param string $username
	 * @return boolean
	 */
	static function is_admin_login($username=""){
		global $current_user;
		$is_admin_login=false;
		if($username===false)return false;
		if(empty($username))$username=$current_user;
		if(is_object($username)){
			if($username == $current_user && function_exists('is_super_admin') && is_super_admin()) $is_admin_login=true;
			elseif(!empty($username->roles)&& in_array("administrator",$username->roles)) $is_admin_login=true;
			elseif(!empty($username->user_level)&& $username->user_level>7)$is_admin_login=true;
		}elseif(is_string($username)){
			if($current_user->user_login == $username) $udata=$current_user;
			else $udata=get_user_by("login",$username);
			if($udata == $current_user && function_exists('is_super_admin') && is_super_admin()) $is_admin_login=true;
			elseif(!empty($udata->roles)&& in_array("administrator",$udata->roles)) $is_admin_login=true;
			elseif(!empty($udata->user_level)&& $udata->user_level>7) $is_admin_login=true;
		}
		return $is_admin_login;
	} //end is_admin_login

	/**
	 * Verify that wassup is network-activated in Wordpress multisite.
	 * @since v1.9
	 * @param none
	 * @return boolean
	 */
	public function network_activated_plugin($plugin_file=""){
		global $wpdb;
		$is_network_activated=false;
		$wassupplugin=plugin_basename(WASSUPDIR."/wassup.php");
		if(empty($plugin_file)) $plugin_file=$wassupplugin;
		if(is_multisite()){
			$plugins=get_site_option('active_sitewide_plugins');
			if(isset($plugins[$plugin_file])){
				$is_network_activated=true;
			}elseif($plugin_file == $wassupplugin){
				$network_settings=get_site_option('wassup_network_settings');
				if(!empty($network_settings['wassup_table']) && $network_settings['wassup_table']==$wpdb->base_prefix."wassup") $is_network_activated=true;
			}
		}
		return $is_network_activated;
	}
	/**
	 * Return the user capabilities string equivalent of a user level number.
	 * @since v1.9
	 * @param integer $userlevel
	 * @return string
	 */
	public function get_access_capability($userlevel=""){
		if($userlevel=="" || !is_numeric($userlevel)){
			$userlevel=$this->wassup_userlevel;
		}
		$access='read';	//default
		if(is_numeric($userlevel)){
			if($userlevel >=8) $access='manage_options';	//Admin
			elseif($userlevel >=6) $access='publish_pages';	//Editor
			elseif($userlevel >=2) $access='publish_posts';	//Author
			elseif($userlevel >=1) $access='edit_posts';	//Contributor
			else $access='read';
		}
		return $access;
	}

	/** Set hash value for wassup ajax requests and cookies */
	static function get_wp_hash($hashkey="") {
		$wassuphash="";
		if (empty($hashkey)) {
			//v1.9 bugfix: 'secret_key' obsolete, replaced with 'auth_key'
			if(defined('AUTH_KEY'))$hashkey=AUTH_KEY;	//in WP 3.X
			elseif(defined('SECRET_KEY'))$hashkey=SECRET_KEY; //in WP 2.X
			else $hashkey="wassup-".sprintf('%03d',rand(0,999));
			//for multisite, append subsite_id to hashkey
			if(is_multisite() && !is_network_admin()) $hashkey .= $GLOBALS['current_blog']->blog_id;
		}
		$wassuphash=wp_hash($hashkey);
		return $wassuphash;
	}
	/**
	 * Convert associative array to this class object
	 * @since 1.8
	 * @param array (assoc)
	 * @return boolean
	 */
	private function _options2class($options_array) { 
		if(!empty($options_array) && is_array($options_array)){
		foreach ($options_array as $o_key => $o_value){
			if(isset($this->$o_key)){ //returns false for null values
				$this->$o_key = $o_value;
			}elseif(function_exists('property_exists')){	//PHP 5.1+ function
				if(property_exists($this,$o_key)){
					$this->$o_key = $o_value;
				}
			}elseif(array_key_exists($o_key,$this)){ //valid for objects in PHP 4.0.7 thru 5.2 only
				$this->$o_key = $o_value;
			}
		} //end foreach
		}else{
			return false;
		}
		return true;
	} //end _options2class

	/** display a system notice or message in admin panel. */
	public function showMessage($message="") {
		global $wp_version,$current_user;
		if(empty($message) && (empty($_GET['page'])|| stristr($_GET['page'],'wassup')!==false) && !empty($this->wassup_alert_message)) $message=$this->wassup_alert_message;
		//New in v1.9: show user-specific alert messages
		if(empty($message)){
			if(!is_object($current_user) || empty($current_user->ID)) wp_get_current_user();
			$wassup_user_settings = get_user_option('_wassup_settings',$current_user->ID);
			if(!empty($wassup_user_settings['ualert_message'])) $message=$wassup_user_settings['ualert_message'];
			$wassup_user_settings['ualert_message']="";
			update_user_option($current_user->ID,'_wassup_settings',$wassup_user_settings);
		}
		if(!empty($message)){
			$error=__("error","wassup");
			$warning=__("warning","wassup");
			$updated=__("updated","wassup");
			$upgraded=__("upgraded","wassup");
			$deleted=__("deleted","wassup");
			$mclass="fade updated";
			if(stristr($message,$error)!==false){
				if(version_compare($wp_version,'4.1','>=')) $mclass="notice error";
				else $mclass="fade error";
			}elseif(version_compare($wp_version,'4.1','>=')){
				if(stristr($message,$warning)!==false) $mclass="notice notice-warning";
				elseif(stristr($message,$updated)!==false || stristr($message,$upgraded)!==false || stristr($message,$deleted)!==false) $mclass="notice updated";
				else $mclass="notice notice-info";
			}
			//display message prefaced with version #
			echo '<div id="wassup-message" class="'.$mclass.'">WassUp '.WASSUPVERSION.': '.esc_attr($message);
			echo '</div>'."\n";
			if($message == $this->wassup_alert_message){
				$this->clearMessage();
			}
		}
	}
	/** displays an alert (error) message. */
	public function showError($message="") {
		$this->showMessage($message);
	}
	/** remove a notice from wassup_alert_message. @since v1.9 */
	public function clearMessage($message_text=""){
		if(!empty($message_text)){
			if($this->wassup_alert_message==$message_text){
				$this->wassup_alert_message="";
				$this->saveSettings();
			}elseif(stristr($this->wassup_alert_message,$message_text)!==false){
				$this->wassup_alert_message="";
				$this->saveSettings();
			}
		}else{
			$this->wassup_alert_message="";
			$this->saveSettings();
		}
	}
} //end class wassupOptions
} //end if !class_exists

//New in v1.9: wassupDb static class for WassUp table operations
if(!class_exists('wassupDb')){
/**
 * Static class for WassUp table operations and data caching.
 *
 * @since v1.9
 * @author Helene D. <http://helenesit.com>
 */
class wassupDb{
	/** Private constructor for true static class - prevents direct creation of object */
	private function __construct(){}

	/** Verify that a table exists in wordpress db */
	static function table_exists($table){
		global $wpdb;
		$is_table=false;
		if(!empty($table)&& preg_match('/^([a-z0-9\-_\.]+)$/i',$table)>0){
			if(method_exists($wpdb,'esc_like')) $sql=sprintf("SHOW TABLES LIKE '%s'",$wpdb->esc_like($table));
			else $sql=sprintf("SHOW TABLES LIKE '%s'",like_escape($table));
			$result=$wpdb->get_var($sql);
			if(!empty($result) && !is_wp_error($result)){
				if($result == $table) $is_table=true;
			}
		}
		return $is_table;
	} //table_exists

	/**
 	 * Perform a "table status" query and cache the result.
	 *
	 *  - table_status is cached for up to 1 hour to avoid overusing this costly mysql operation
	 * @param string
	 * @return object
	 */
	static function table_status($table){
		global $wpdb,$wdebug_mode;
		$table_status=false;
		if(empty($table) || !self::table_exists($table)){
			$error_msg= __FUNCTION__." failed - missing table name";
		}else{
			$meta_key='_table_status';
			$table_status=self::get_wassupmeta($table,$meta_key);
			if(empty($table_status)){
				$table_status=$wpdb->get_row(sprintf("SHOW TABLE STATUS LIKE '%s'",self::esc_like($table)));
				if(is_wp_error($table_status)){
					$error_msg=" table_status error# ".$table_status->get_error_code().": ".$table_status->get_error_message();
					$table_status=false;
				}elseif(!empty($table_status)){
					$expire=time()+3602;
					$cache_id=self::update_wassupmeta(esc_attr($table),$meta_key,$table_status,$expire);
				}else{
					$table_status=false;
				}
			}
		}
		if(!empty($error_msg)){
			if($wdebug_mode) echo "\n<!-- ".__CLASS__." ERROR: ".$error_msg." -->";
		}
		//always return an object
		if(empty($table_status) || !is_object($table_status)){
			if(empty($table_status['Data_length'])) $tstatus=array('Rows'=>0,'Data_length'=>0,'Index_length'=>0,'Engine'=>"",'Type'=>"");
			else $tstatus=$table_status;
			$table_status=(object)$tstatus;
		}
		return $table_status;
	} //table_status

	/**
 	 * Insert a record into a wassup table.
	 *
	 *  - insert record is an associative array
	 *  - a record id number is returned on success.
	 * @param string $table, array $new_record, boolean $delayed
	 * @return string
	 */
	static function table_insert($table,$new_record,$delayed=false){
		global $wpdb,$wassup_options,$wdebug_mode;
		$rec_id=false;
		$error_msg="";
		if(empty($wassup_options->delayed_insert)) $delayed=false;
 		//table and record must be specfied
		if(empty($table)|| empty($new_record)){
			$error_msg=__FUNCTION__." failed - missing table or record parameter";
		}elseif(!is_array($new_record)){
			//record must be array
			$error_msg=__FUNCTION__." failed - insert record NOT an array $new_record";
		}
		if(!empty($error_msg)){
			$error=new WP_Error('1',$error_msg);
			return $error;
		}
		//insert the record
		if(!$delayed && method_exists($wpdb,'insert')){
			//insert with 'wpdb::insert' method 
			$rec_id=$wpdb->insert($table,$new_record);
		}else{
			//insert with 'wpdb::query' for delayed insert
			$cols="";
			$vals="";
			$i=0;
			foreach($new_record AS $field=>$val){
				if(is_numeric($val))$value=$val;
				else $value=self::sanitize($val,true);
				if($i==0){
					$cols="`$field`";
					$vals="$value";
				}else{
					$cols .=", `$field`";
					$vals .=", $value";
				}
				$i++;
			} //end foreach
			//delayed insert option for myISAM and ISAM tables
			if($delayed) $delayed="DELAYED";
			else $delayed="";
			$rec_id=$wpdb->query(sprintf("INSERT $delayed INTO $table (%s) VALUES (%s)",$cols,$vals));
		}
		return $rec_id;
	} //end table_insert

	/**
	 * perform a delete operation on a table
	 * @param string, string
	 * @return integer (no. deleted)
	 */
	static function table_delete($wtable="",$where_condition){
		global $wpdb,$wassup_options,$wdebug_mode;
		$deleted=false;
		if(!empty($where_condition)){
			if(empty($table))$table=$wassup_options->wassup_table;
			if(self::table_exists($table) && stristr($where_condition,'where')!==false){
				$sql=sprintf("DELETE FROM %s %s",$table,$where_condition);
				$deleted=$wpdb->query($sql);
				if(is_wp_error($deleted)){
					$error_msg=' &nbsp; WP_Error in '.__FUNCTION__.' '.$deleted->get_error_message()." \n<br/>SQL=\$wpdb->query($sql)";
					$deleted=0;
				}
			}else{
				$error_msg=' &nbsp; Error in '.__FUNCTION__.' missing table or bad where condition';
			}
		}else{
			$error_msg=' &nbsp; Error in '.__FUNCTION__.' missing where condition';
		}
		if(!empty($error_msg)){
			if($wdebug_mode) echo "\n<!-- ".__CLASS__." ERROR: ".$error_msg." -->";
		}
		return $deleted;
	}

	/**
	 * Deletes "cached" records from 'wp_wassup_meta' table.
	 * - deletes records matching meta_key==parameter or 
	 * - deletes all records starting with "_" (underscore) when empty parameter
	 * @param string $meta_key
	 * @return boolean
	 */
	static function clear_cache($meta_key=""){
		global $wpdb,$wassup_options,$wdebug_mode;
		$cleared=false;
		$cache_table=$wassup_options->wassup_table ."_meta";
	 	// deletes all records starting with "_" (underscore)
		if(empty($meta_key)){
			$sql=sprintf("DELETE FROM %s WHERE `meta_key` LIKE '\\_%%' AND `meta_expire`>0",$cache_table);	//TODO: verify that escaped '_' requires 2 backslashes, not 1
			$deleted=$wpdb->query($sql);
			if(is_wp_error($deleted)){
				$error_msg=' &nbsp; clear_cache error# '.$deleted->get_error_code()." ".$deleted->get_error_message()." \n<br/>SQL=\$wpdb->query($sql)";
				$deleted=0;
			}
	 	//delete records matching meta_key==parameter
		}elseif(preg_match('/^([0-9a-z_\-\.]+)$/',$meta_key)>0){
			$deleted=self::delete_wassupmeta('','*',$meta_key);
		}
		if(!empty($deleted)&& is_numeric($deleted))$cleared=true;
		if(!empty($error_msg)){
			if($wdebug_mode) echo "\n<!-- ".__CLASS__." ERROR: ".$error_msg." -->";
		}
		return $cleared;
	} //end clear_cache

	/**
	 * verify that a table can be optimized.
	 * - table engines: MyISAM, ARCHIVE, or InnoDb with "innodb_file_per_table" option enabled can be optimized.
	 * @param string (table_name)
	 * @return boolean
	 */
	static function is_optimizable_table($table_name=""){
		global $wpdb,$wassup_options;
		$is_optimizable=false;
		$tengine="";
		if(empty($table_name))$table_name=$wassup_options->wassup_table;
		if(!empty($table_name)){
			$tstatus=self::table_status($table_name);
			if(is_object($tstatus) && isset($tstatus->Engine)){
				$tengine=strtolower($tstatus->Engine);
				if(empty($tengine)|| $tengine=="myisam" || $tengine=="archive" || $tengine=="isam"){
					$is_optimizable=true;
				}elseif($tengine=="innodb"){
					$innodb_optimizable=$wpdb->get_var("SELECT @@global.innodb_file_per_table AS innodb_optimizable FROM DUAL");
					if(!empty($innodb_optimizable)&& !is_wp_error($innodb_optimizable) && ($innodb_optimizable=="1" || strtolower($innodb_optimizable)=="on")) $is_optimizable=true;
				}
			}
		}
		return $is_optimizable;
	} //end is_optimizable_table

	/**
	 * method for PHP5.5 and Mysqli compatibility.
	 * @todo - remove all references to old mysql_client_encoding and delete this method
	 */
	static function mysql_client_encoding(){
		global $wpdb;
		if (empty($wpdb->use_mysqli)) return mysql_client_encoding();
		else return mysqli_character_set_name();
	}

	/**
	 * Save wassup data to wp_wassup_meta table for caching.
	 *  - records are cached with a default expiration of 24 hours, if none is given as argument
	 * @param mixed(4): string(2) $wassup_key $meta_key, string/array $metavalue, integer $expire (as time)
	 * @return string $meta_id
	 */
	static function save_wassupmeta($wassup_key,$meta_key,$metavalue,$expire=false){
		global $wpdb,$wassup_options,$wdebug_mode;
		$table=$wassup_options->wassup_table.'_meta';
		$meta_id=false;
		//wassup_key or meta_key must be specified
		if(empty($wassup_key)&& empty($meta_key)){
			$error_msg=__FUNCTION__." failed! - missing both 'wassup_key' and 'meta_key' parameters";
			if($wdebug_mode)echo "\n<!-- ".__CLASS__." ERROR: ".$error_msg." -->";
			return false;
		}
		//24-hour expire default... rounded to minute
		if($expire===false || !is_numeric($expire))$expire= (ceil(time()/60))*60+24*3600;
		//serialize $metavalue array
		if(is_array($metavalue)|| is_object($metavalue))$meta_value=maybe_serialize($metavalue);
		else $meta_value=$metavalue;
		$meta_record=array('wassup_key'=>$wassup_key,
				   'meta_key'=>$meta_key,
				   'meta_value'=>"$meta_value",
				   'meta_expire'=>(int)$expire);
		//save to table
		$meta_id=self::table_insert($table,$meta_record);
		return $meta_id;
	} //end save_wassupmeta

	/**
	 * Update existing cache data in 'wp_wassup_meta' table.
	 * @param mixed(4): string(2) $wassup_key $meta_key, string/array $metavalue, integer $expire (as time)
	 * @return string $meta_id
	 */
	static function update_wassupmeta($wassup_key,$meta_key,$metavalue,$expire=false){
		global $wpdb,$wassup_options,$wdebug_mode;
		$table=$wassup_options->wassup_table.'_meta';
		$meta_id=false;
		$error_msg="";
		//both wassup_key and meta_key must be specified for update
		if(empty($wassup_key)|| empty($meta_key)){
			$error_msg=__FUNCTION__." failed! - missing either 'wassup_key' or 'meta_key' parameter";
		}else{
			//get meta_id of existing record, if any
			$result=$wpdb->get_var(sprintf("SELECT `meta_id` FROM $table WHERE `wassup_key`='%s' AND `meta_key`='%s' LIMIT 1",esc_attr($wassup_key),esc_attr($meta_key)));
			//update the record (or insert new)
			if(!empty($result) && !is_wp_error($result)) $meta_id=$result;
			if(empty($meta_id)){
				$meta_id=self::save_wassupmeta($wassup_key,$meta_key,$metavalue,$expire);
			}else{
				//24-hour expire default... rounded to minute
				if($expire===false || !is_numeric($expire))$expire=(ceil(time()/60))*60+24*3600;
				//serialize $metavalue array
				if(is_array($metavalue)|| is_object($metavalue))$meta_value=maybe_serialize($metavalue);
				else $meta_value=$metavalue;
				$qry=sprintf("UPDATE `$table` SET `meta_value`='%s', `meta_expire`=%d WHERE `meta_id`=%d",$meta_value,$expire, $meta_id);
				if(!empty($qry)){
					$result=$wpdb->query($qry);
					if(is_wp_error($result)) $error_msg=' &nbsp; update_wassupmeta Error#'.$result->get_error_code().': '.$result->get_error_message()." \n<br/>SQL=\$wpdb->query($qry)";
				}
			}
		}
		if(!empty($error_msg)){
			if($wdebug_mode)echo "\n<!-- ".__CLASS__." ERROR: ".$error_msg." -->";
		}
		return $meta_id;
	} //end update_wassupmeta

	/**
	 * Retrieve an unexpired 'meta_value' from wassup_meta table or an array of 'meta_value's for multiple matching records.  
	 * -records with 'meta_expire' timestamp older than the current time are not returned.
	 * -optional parameter to use 'SQL_NO_CACHE' to force mySQL to lookup data instead of using internal query cache
	 * @param string (2), boolean
	 * @return mixed (string or array)
	 */
	static function get_wassupmeta($wassup_key,$meta_key="",$sql_nocache=false){
		global $wpdb,$wassup_options,$wdebug_mode;
		if(empty($wassup_options))$wassup_options=new wassupOptions;
		$meta_table=$wassup_options->wassup_table.'_meta';
		$meta_value=false;
		$sql_no_cache="";
		$expired=time()-30;
		$sql="";
		$result=false;
		if(!empty($sql_nocache)) $sql_nocache="SQL_NO_CACHE";
		else $sql_nocache="";
		//check that we have a wassup_meta table
		if(!self::table_exists($meta_table)){
			$error_msg=__FUNCTION__." failed - table $meta_table not found!";
		//check for matching wassup_key
		}elseif(!empty($wassup_key)){
			if(!empty($meta_key)){
				$sql=sprintf("SELECT $sql_nocache `meta_value` FROM `$meta_table` WHERE `wassup_key`='%s' AND `meta_key`='%s' AND (`meta_expire`=0 OR `meta_expire`>%d) ORDER BY `meta_expire` DESC LIMIT 1",$wassup_key,$meta_key,$expired);
				//$result=$wpdb->get_var($sql);
				$result=$wpdb->get_results($sql);
				if(!empty($result) && !is_wp_error($result) && !empty($result[0]->meta_value)) $meta_value=$result[0]->meta_value;
			}else{
				//return an array of all results with same wassup_key
				$sql=sprintf("SELECT $sql_nocache `meta_value` FROM `$meta_table` WHERE `wassup_key`='%s' AND (`meta_expire`=0 OR `meta_expire`>%d)",$wassup_key,$expired);
				$result=$wpdb->get_col($sql);
				if(!empty($result) && !is_wp_error($result)) $meta_value=$result;
			}
		//check for matching meta_key
		}elseif(!empty($meta_key)){
			//return an array of all results with same meta_key
			$sql=sprintf("SELECT $sql_nocache `meta_value` FROM `$meta_table` WHERE `meta_key`='%s' AND (`meta_expire`=0 OR `meta_expire`>%d)",$meta_key,$expired);
			$result=$wpdb->get_col($sql);
			if(!empty($result) && !is_wp_error($result)) $meta_value=$result;
		}
		//unserialize arrays/objects before output
		if(!empty($meta_value) && !is_array($meta_value)&& !is_object($meta_value)){
			$results=maybe_unserialize(html_entity_decode($meta_value));
			if(is_array($results)|| is_object($results))$meta_value=$results;
		}
		if($wdebug_mode){
			if(!empty($result) && is_wp_error($result)){
				$errno=$result->get_error_code();
				if($errno >0) $error_msg=__FUNCTION__.' query failed - MySQL error#'.$errno.' '.$result->get_error_message()." \n<br/>SQL=$sql";
				echo "\n<!-- ".__CLASS__." ERROR: ".$error_msg." -->";
			}
		}
		return $meta_value;
	} //end get_wassupmeta

	/**
	 * Delete a 'wp_wassup_meta' record.
	 *  - single or multiple records may be deleted by 'meta_id', or 'wassup_key', or 'meta_key' regardless of expire timestamp
	 *  - 'wassup_key' and 'meta_key' arguments are both required when 'meta_id'=null
	 *  - wildcards (*) character can be used for bulk delete by either 'meta_key' or 'wassup_key', but not both.
	 *  USAGE: meta_id|"", [(wassup_key|*),(meta_key|*))].
	 *
	 * @param mixed (integer $meta_id, string $wassup_key, string $meta_key)
	 * @return integer
	 */
	static function delete_wassupmeta($meta_id,$wassup_key="",$meta_key=""){
		global $wpdb,$wassup_options,$wdebug_mode;
		$meta_table=$wassup_options->wassup_table.'_meta';
		$rows_affected=0;
		$deleted=0;
		$sql="";
		//prepare the sql delete statement from function parameters
		if(self::table_exists($meta_table)){
			if(!empty($meta_id)){
				$sql=sprintf("DELETE FROM $meta_table WHERE `meta_id`=%d",$meta_id);
			}elseif(!empty($wassup_key)&& !empty($meta_key)){
				if($wassup_key !="*" && $meta_key !="*"){
					$sql=sprintf("DELETE FROM $meta_table WHERE `meta_key`='%s' AND `wassup_key`='%s'",esc_attr($meta_key),esc_attr($wassup_key));
				}elseif($meta_key !="*"){
					$sql=sprintf("DELETE FROM $meta_table WHERE `meta_key`='%s'",esc_attr($meta_key));
				}elseif($wassup_key !="*"){
					$sql=sprintf("DELETE FROM $meta_table WHERE `wassup_key`='%s'",esc_attr($wassup_key));
				}
			}else{
				$error_msg=__FUNCTION__.' failed - bad or missing arguments!'."&nbsp; (meta_id=".esc_attr($meta_id)." , &nbsp;wassup_key=".esc_attr($wassup_key)." , &nbsp;meta_key=".esc_attr($meta_key).")";
			}
		}else{
			$error_msg=__FUNCTION__.' failed - table '.$meta_table.' does not exist!';
		}
		//do the delete
		if(!empty($sql)){
			$deleted=$wpdb->query($sql);
			$errno=0;
			if(is_wp_error($deleted)){
				$errno=$deleted->get_error_code();
				if($errno >0)$error_msg=' delete_wassupmeta failed with MySQL error#'.$errno.' '.$deleted->get_error_message(). " \n<br/>SQL=$sql";
			}
			$deleted=$wpdb->rows_affected+0;
		}
		if(!empty($error_msg)){
			if($wdebug_mode) echo "\n<!-- ".__CLASS__." ERROR: ".$error_msg." -->";
		}
		return $deleted;
	} //end delete_wassupmeta

	/**
	 * clean data for insertion into mySQL.
	 *  - to prevent SQL injection - renamed from wSanitizeData
	 *  - an alternative to "wpdb::prepare" for older WP versions.
	 * @since WassUp 1.7 (as wSanitizeData)
	 * @param string $var, boolean $quotes
	 * @return string
	 */
	static function sanitize($var,$quotes=false){
		global $wpdb;
		//clean strings and add quotes
		if(is_string($var)){
			$varstr=stripslashes($var);
			//sanitize urls separately
			if(strpos($varstr,'://')!==false){
				if(function_exists('esc_url_raw'))$varstr=esc_url_raw($varstr);
				else $varstr=esc_url($varstr,null,'db');
				//oops, clean_url chomp
				if(empty($varstr))$varstr=esc_attr(stripslashes($var));
			}else{
				$varstr=esc_sql($varstr);
			}
 			if($quotes)$var="'". $varstr ."'";
			else $var=$varstr;
		//convert boolean variables to binary boolean
		}elseif(is_bool($var)&& $quotes){
			$var=($var)?1:0;
		//convert null variables to SQL NULL
		}elseif(is_null($var)&& $quotes){
			$var="NULL";
		}
		//note numeric values do not need to be sanitized
		return "$var";
	} //end sanitize

	/**
	 * Escape special characters for use in an SQL 'like' query.
	 *  - uses 'like_escape' or 'wpdb::esc_like' depending on Wordpress version
	 * @param string $sqlstring
	 * @return string
	 */
	static function esc_like($sqlstring){
		global $wpdb;
		$escaped_string=$sqlstring;
		if(!empty($sqlstring)){
			if(method_exists($wpdb,'esc_like')) $escaped_string= $wpdb->esc_like($sqlstring);
			else $escaped_string=like_escape($sqlstring);
		}
		return $escaped_string;
	}

	/**
	 * Return an MySQL system variable value.
	 * @todo find cause of mysql timeout error when looking up dbengine for wp_wassup table
	 * @since 1.7.2 (as wassupOptions::getMySQLsetting)
	 * @param string
	 * @return string
	 */
	static function get_db_setting($mysql_var,$mysql_table="") {
		global $wpdb,$wassup_options,$wdebug_mode;
		$mysql_value=false;
		$error_msg="";
		if(empty($mysql_table)) $mysql_table=$wassup_options->wassup_table;
		//get the table storage engine
		if($mysql_var=="engine" || $mysql_var=="dbengine"){
			if(!empty($mysql_table)){
				//New in v1.9: replaced cpu-intensive "show table status" query with "show create table" for faster table engine lookup 
				$sql="SHOW CREATE TABLE $mysql_table";
				$result=$wpdb->get_results($sql,ARRAY_N);
				if(!empty($result)){
					if(!is_wp_error($result)){
						if(!empty($result[0][1]) && preg_match('/\sENGINE\=(\w+)\s/',$result[0][1],$pcs)>0) $mysql_value=$pcs[1];
					}else{
						$errno=$result->get_error_code();
						if($errno >0) $error_msg=' get_db_setting('.$mysql_var.') failed with MySQL error#'.$errno.' '.$myresult->get_error_message()."\n";
					}
				}
			}
			//default to db storage engine when no table
			if(empty($mysql_value)) {
				$result=$wpdb->get_results("SHOW VARIABLES LIKE 'storage_engine'",ARRAY_A);
				if(!empty($result) && !is_wp_error($result) && !empty($result[0]->Value)) $mysql_value=$result[0]->Value;
			}
		//get the timezone 
		}elseif ($mysql_var == "timezone") {
			$sql_timezone=false;
			$sql_sys_timezone="";
			$result=$wpdb->get_results("SHOW VARIABLES LIKE '%zone'");
			if(!is_wp_error($result)){
			foreach ($result as $col) {
				if ($col->Variable_name == "system_time_zone") {
					$sql_sys_timezone=$col->Value;
				} elseif ($col->Variable_name == "time_zone") {
					$sql_timezone=$col->Value;
				} elseif ($col->Variable_name == "timezone") {
					$sql_timezone=$col->Value;
				}
			}
			if ($sql_timezone == "SYSTEM" || empty($sql_timezone)) {
				$host_timezone=$wassup_options->getHostTimezone();
				if (!empty($host_timezone)) {
					$sql_timezone=$host_timezone;
				} else {
					$sql_timezone=$sql_sys_timezone;
				}
			}
			}
			if (!empty($sql_timezone)) $mysql_value=$sql_timezone;
		//get timezone offset for today's date.
		} elseif ($mysql_var == "tzoffset") {
			$tzoffset=false;
			$result=$wpdb->get_var("SELECT NOW() AS tnow FROM DUAL");
			//calculate mysql timezone offset by converting MySQL's NOW() to a timestamp and subtracting UTC current time from it. 
			// Note: conversion to timestamp is affected by PHP TZ setting, so remove that offset from calculation
			if(!empty($result) && !is_wp_error($result)){
				$now=((int)(time()/1800))*1800; //truncate to 1/2 hour
				if(function_exists('date_timestamp_get')){
					$mysql_dt=new DateTime($result);
					$mysql_time=$mysql_dt->getTimestamp();
					$adjust=$mysql_dt->getOffset();
				}else{
					$mysql_time=strtotime($result);
					$adjust=(int)date('Z');
				}
				$tzoffset=((int)($mysql_time/1800))*1800 - $now;
				if(is_numeric($adjust)) $tzoffset +=$adjust;
				//change offset to MySQL "[+-]hh:mm" format
				$mysql_value=self::format_tzoffset($tzoffset);
			}
		//get a mysql variable with parameter name
		} elseif(!empty($mysql_var)) { 
			$result=$wpdb->get_results(sprintf("SHOW VARIABLES LIKE '%s'",self::esc_like(esc_attr($mysql_var))));
			if (!is_wp_error($result) && !empty($result)) {
			foreach ($result as $col) {
				if ($col->Variable_name == $mysql_var) {
					$mysql_value=$col->Value;
					break 1;
				}
			}
			}
		}
		if($wdebug_mode){
			if(!empty($result) && is_wp_error($result)){
				$errno=$result->get_error_code();
				if($errno >0) $error_msg .=' get_db_setting('.$mysql_var.') failed with MySQL error#'.$errno.' '.$result->get_error_message();
			}
			if(!empty($error_msg)) echo "\n<!-- ".__CLASS__." ERROR: ".$error_msg." -->";
		}
		return $mysql_value;
	} //end get_db_setting

	/**
	 * Convert an offset to MySQL "[+-]hh:mm" format.
	 *  - offset is converted from seconds or hours to MySQL "[+-]hh:mm" format.
	 * @since 1.8 (as wassupOptions::formatTimezoneOffset)
	 * @param string
	 * @return string
	 */
	static function format_tzoffset($offset=false) {
		$tzoffset=false;
		if(preg_match('/^[\-+]?[0-9\.]+$/',$offset)>0){ //must be a number
			//convert seconds to hours:minutes
			$n=false;
			if($offset > 12 || $offset < -12) $noffset=$offset/3600;
			else $noffset=$offset;
			$n = strpos($noffset,'.');
			if($n !== false){
				$offset_hrs=substr($noffset,0,$n);
				$offset_min=(int)substr($noffset,$n+1)*6;
			}else{
				$offset_hrs=$noffset;
				$offset_min=0;
			}
			if($offset < 0) $tzoffset=sprintf("%d:%02d",$offset_hrs,$offset_min);
			else $tzoffset="+".sprintf("%d:%02d",$offset_hrs,$offset_min);
		}elseif(preg_match('/^([\-+])?(\d{1,2})?\:(\d{2})/',$offset,$match)>0){
			if(empty($match[2])) $match[2]="0";
			if(!empty($match[1]) && $match[1]=="-") $tzoffset="-".sprintf("%d:%02d",$match[2],$match[3]);
			else $tzoffset="+".sprintf("%d:%02d",$match[2], $match[3]);
		}
		return $tzoffset;
	} //end format_tzoffset

	/**
	 * Perform scheduled/delayed db operations on wassup tables.
	 *  - for use with 'wp_schedule_event' hook.
	 * @param array
	 * @return void
	 */
	static function scheduled_dbtask($args=array()){
		global $wpdb,$wdebug_mode;
		//get dbtasks argument
		$dbtasks=array();
		if(!empty($args)){
			if(!is_array($args)) $args=maybe_unserialize($args);
			if(isset($args['dbtasks'])){
				extract($args);
			}elseif(is_array($args)){
				$dbtasks=$args;
			}else{
				$dbtasks[]=$args;
			}
		}
		if(!empty($dbtasks) && !is_array($dbtasks)){
			$args=maybe_unserialize($dbtasks);
			if(is_array($args)) $dbtasks=$args;
		}
		$wassup_settings=get_option('wassup_settings');
		$table_prefix=$wassup_settings['wassup_table'];
		$affected_recs=0;
		$dbtask_errors=array();
		if(!empty($dbtasks) && is_array($dbtasks)){
			//some db operations can be slow on large tables, so extend script execution time up to 30 minutes
			if(!ini_get('safe_mode')) @set_time_limit(1800);
			//increase mysql session timeout to 15 minutes
			$mtimeout=$wpdb->get_var("SELECT @@session.wait_timeout AS mtimeout FROM DUAL");
			if(!empty($mtimeout) && !is_wp_error($mtimeout) && is_numeric($mtimeout) && $mtimeout< 900){
				$result=$wpdb->query("SET wait_timeout=900");
			}
		foreach($dbtasks as $db_sql){
			$results=false;
			$error_msg="";
			//limit allowed sql to certain tasks and to Wassup tables only
			if(strpos($db_sql,"DELETE FROM `$table_prefix")!==false){
				$results=$wpdb->query($db_sql);
			}elseif(strpos($db_sql,"UPDATE LOW_PRIORITY `$table_prefix")!==false){
				$results=$wpdb->query($db_sql);
			}elseif(strpos($db_sql,"UPDATE `$table_prefix")!==false){
				$results=$wpdb->query($db_sql);
			}elseif(strpos($db_sql,"OPTIMIZE TABLE `$table_prefix")!==false){
				$results=$wpdb->query($db_sql);
			}else{
				//bad dbtask, create an error record
				$error_msg =" ERROR: Unknown task ".esc_attr($db_sql);
			}
			//check for errors in mysql results
			if(!empty($results) && is_wp_error($results)){
				$dbtask_errors[]=" ERROR: wpdb error#".$results->get_error_code().": ".$results->get_error_message()." for sql=$db_sql";
			}elseif(!empty($error_msg)){
				$dbtask_errors[]=$error_msg;
			}else{
				$affected_recs += $wpdb->rows_affected + 0;
			}
		} //end foreach
		} //end if dbtasks
		if(!empty($wdebug_mode)){
			//email error output from cron
			$message="";
			if(!empty($dbtask_errors)){
				$subject=sprintf(__("%s Error!","wassup"),'Wassup wp-cron');
				$message ="scheduled dbtask encountered an error:\n";
				foreach($dbtask_errors AS $error_msg){
					$message .=$error_msg."\n";
				}
			//}else{
			//	$message ="scheduled dbtask completed successfully. Records affected=$affected_recs";
			}
			if(!empty($message)){
				$blogurl = wassupURI::get_sitehome();
				$recipient=get_bloginfo('admin_email');
				$sender='From: '.get_bloginfo('name').' <wassup_noreply@'.parse_url($blogurl,PHP_URL_HOST).'>';
				wp_mail($recipient,$subject,$message,$sender);
			}
		}
		//return $affected_recs; //don't return anything
	} //end scheduled_dbtask

	/**
	 * cleanup wassup temporary records (hourly via wp-cron):
	 *  - delete inactive records from wassup_tmp
	 *  - delete expired cache records from wassup_meta
	 *
	 * Inactive wassup_tmp records vary by visitor type: 
	 * - logged-in users are inactive after 10 minutes
	 * - anonymous users are inactive after 3 minutes
	 * - spiders are inactive after 1 minute
	 * @since v1.9
	 * @return void
	 */
	static function temp_cleanup(){
		global $wpdb,$wassup_options;
		$wassup_table=$wassup_options->wassup_table;
		$wassup_tmp_table=$wassup_table . "_tmp";
		$wassup_meta_table=$wassup_table . "_meta";
		$timestamp=current_time('timestamp');
		$timenow=(int)time();
		//delete inactive records from wassup_tmp table
		$result=$wpdb->query(sprintf("DELETE FROM `%s` WHERE `timestamp`<'%d' OR (`timestamp`<'%d' AND `username`='') OR (`timestamp`<'%d' AND `spider`!='')",$wassup_tmp_table,(int)$timestamp - 10*60,(int)$timestamp - 3*60,(int)$timestamp - 60));
		//delete expired cache records from wassup_meta
		$result=$wpdb->query(sprintf("DELETE FROM `%s` WHERE `meta_expire`>'0' AND `meta_expire`<'%d'",$wassup_meta_table,$timenow - 3600));
	}
	/**
	 * Do automatic purge of old records from wassup table (daily via wp-cron)
	 * @since v1.9
	 */
	static function auto_cleanup(){
		global $wpdb,$wassup_options;
		$deleted=0;
		//do purge of old records
		if(!empty($wassup_options->delete_auto) && $wassup_options->delete_auto!="never"){
			$wassup_table=$wassup_options->wassup_table;
			$wassup_meta_table=$wassup_table . "_meta";
			$timestamp=current_time('timestamp');
			$timenow=(int)time();
			//use visit timestamp not current time for delete
			$delete_from= @strtotime($wassup_options->delete_auto,$timestamp);
			$delete_filter=""; 
			$rows=0;
			if(is_numeric($delete_from) && $delete_from < $timestamp){
				$delete_condition="`timestamp`<'$delete_from'";
				//check for delete filters
				if(!empty($wassup_options->delete_filter)){
				if($wassup_options->delete_filter!="all"){
					$delete_filter=$wassup_options->getFieldOptions("delete_filter","sql",$wassup_options->delete_filter);
					if(!is_string($delete_filter)) $delete_filter="";
				}else{
					$delete_filter="";
				}
				}
				$multisite_whereis="";
				if($wassup_options->network_activated_plugin() && !empty($GLOBALS['current_blog']->blog_id)){
					$multisite_whereis = sprintf(" AND `subsite_id`=%d",(int)$GLOBALS['current_blog']->blog_id);
				}
				$delete_filter .= $multisite_whereis;
				$result=$wpdb->get_var(sprintf("SELECT COUNT(`id`) FROM `%s` WHERE `timestamp`<'%d' %s",$wassup_table,$delete_from,$delete_filter));
				if(!empty($result) && !is_wp_error($result) && is_numeric($result)) $rows=$result;

			} //end if delete_from
			//do delete only when there are 50+ records
			if($rows >50){
				$deleted=$wpdb->query(sprintf("DELETE FROM `%s` WHERE `timestamp`<'%d' %s",$wassup_table,$delete_from,$delete_filter));
				//save delete_auto timestamp to prevent multiple auto deletes in 1 day
				if(!empty($deleted) && !is_wp_error($deleted)){
					$expire=time()+24*3600;
					$cache_id=wassupDb::update_wassupmeta($wassup_table,'_delete_auto',$timestamp,$expire);
					//clear table_status from wassup_meta cache after auto delete
					$result=$wpdb->query(sprintf("DELETE FROM `%s` WHERE `wassup_key`='%s' AND `meta_key`='_table_status'",$wassup_table."_meta",$wassup_table));
					//reschedule optimize to run today when bulk delete larger than 1000 records
					if($deleted >1000 && !empty($wassup_optimize)){
						$wassup_optimize=$timenow - 24*60*60;
					}
				}else{
					$deleted=0;
				}
			}
		} //end if delete_auto
		if(!empty($wdebug_mode)){
			//email delete message from cron
			$message="";
			if($deleted > 0){
				$subject=__("Wassup auto-delete notice","wassup");
				$message =sprintf(__("Auto-delete deleted %d old %s table records today.","wassup"),$deleted,$wassup_options->wassup_table);
				$blogurl = wassupURI::get_sitehome();
				$recipient=get_bloginfo('admin_email');
				$sender='From: '.get_bloginfo('name').' <wassup_noreply@'.parse_url($blogurl,PHP_URL_HOST).'>';
				wp_mail($recipient,$subject,$message,$sender);
			}
		}
	} //end auto_cleanup

	/**
	 * Retrieve raw data from wassup table in SQL format for export.
	 *  - taken partially from wp-db-backup plugin by Alain Wolf, Zurich, SW - http://www.ilfilosofo.com/blog/wp-db-backup/
	 *  - a partial export is returned if script execution timelimit of 15 minutes is exceeded
	 *  - the record id of last record exported (last_recid) is cached to 'wassup_meta' table.
	 *
	 * @param string(3) $table, $condition, $segment
	 * @return string ($sql)
	 */
	static function backup_table($table,$condition="",$segment='none'){
		global $wpdb,$current_user,$wassup_options,$wdebug_mode;
		define('ROWS_PER_SEGMENT', 600);
		if(!is_object($current_user) || empty($current_user->ID)){
			$user=wp_get_current_user();
		}
		if(!empty($current_user->ID)){
			$wassup_user_settings=get_user_option('_wassup_settings',$current_user->ID);
		}else{
			$err_msg=__("Export ERROR: login required!","wassup");
			$wassup_options->wassup_alert_message=$err_msg;
			$wassup_options->saveSettings();
			return false;
		}
		if(empty($table))$table=$wassup_options->wassup_table;
		$sql_table_name=$wpdb->get_var(sprintf("SHOW TABLES LIKE '%s'",self::esc_like(esc_attr($table))));
		if(empty($sql_table_name) || $sql_table_name!=$table || is_wp_error($sql_table_name)){
			$err_msg=sprintf(__('Error with TABLE %s: Not found','wassup'), esc_attr($table));
			$wassup_user_settings['ualert_message']=$err_msg;
			update_user_option($current_user->ID,'_wassup_settings',$wassup_user_settings);
			return FALSE;
		}
		$sql="";
		$err_msg=false;
		$did_export=false;
		$timelimit=15*60;
		$mtimeout=60;
		$stimeout=ini_get('max_execution_time');
		$stimer_start=time();
		$row_count=0;
		$last_recid=0;
		$exclude_id=false;
		if(isset($_REQUEST['omit_recid']))$exclude_id=true;
		if(!empty($_REQUEST['expid'])&& is_numeric($_REQUEST['expid']))$backup_id=(int)$_REQUEST['expid'];
		else $backup_id=date('YmdH',current_time('timestamp'));
		//Add comments to each section of export file
		if($segment=='none' || $segment==0){
			$result=$wpdb->get_results(sprintf("SHOW CREATE TABLE `%s`",esc_attr($table)),ARRAY_N);
			if(empty($result[0][1]) || is_wp_error($result)) {
				$err_msg=sprintf(__('Error with "SHOW CREATE TABLE" for %s.','wassup'), esc_attr($table));
			}else{
				$table_create=$result[0][1];
				$first_recid=$wpdb->get_var(sprintf("SELECT MIN(`id`) FROM %s %s",esc_attr($table),$condition));
				if(!is_numeric($first_recid)) $first_recid=0;
				//add comments for table structure section
				$sql .="\n#\n";
				$sql .="# " . sprintf(__('Table structure of table %s','wassup'),esc_attr($table))."\n#\n";
				//replace "CREATE TABLE" with "CREATE TABLE IF NOT EXISTS" and remove AUTO_INCREMENT=NNN value from $table_create
				$sql .=preg_replace(array('/^CREATE\sTABLE\s(IF\sNOT\sEXISTS\s)?/i', '/AUTO_INCREMENT\=\d+\s/i'),array('CREATE TABLE IF NOT EXISTS ',''),$table_create).' ;';
				$sql .="\n#\n";
				//add comments for data section
				$sql .="# Start time: ".date('r',current_time('timestamp'))."	 1st exported ID: $first_recid \n";
				$sql .='# ' . sprintf(__('Data contents of table %s','wassup'),esc_attr($table))."\n#\n";
			}
		}
		//Check for table records
		if(empty($err_msg)){
			$table_structure=$wpdb->get_results(sprintf("SHOW COLUMNS FROM %s",esc_attr($table)));
			if(is_wp_error($table_structure)){
				$errno=$table_structure->get_error_code();
				$err_msg=sprintf(__('Error getting table structure of %s: %s','wassup'), esc_attr($table),$errno.' '.$table_structure->get_error_message());
				$table_structure=false;
			}elseif(false == $table_structure){
				$err_msg=sprintf(__('Error getting table structure of %s','wassup'), esc_attr($table));
			}
			$numrecords=$wpdb->get_var(sprintf("SELECT COUNT(*) FROM %s %s",esc_attr($table), $condition));
			if(empty($numrecords)|| !is_numeric($numrecords)){
				if(is_wp_error($numrecords)){
					$errno=$numrecords->get_error_code();
					$err_msg=sprintf(__('Error with table %s: %s','wassup'), esc_attr($table),$errno.' '.$numrecords->get_error_message());
				}else{
					$err_msg=sprintf(__('Error with table %s: No data','wassup'), esc_attr($table));
				}
				$numrecords=0;
			}
		}
		//ABORT due to table error
		if(!empty($err_msg)){
			$wassup_user_settings['ualert_message']=$err_msg;
			update_user_option($current_user->ID,'_wassup_settings',$wassup_user_settings);
			return FALSE;
		}
		//Extend php script timeout to 10 minutes
		//TODO: 'safe_mode' deprecated
		if(!@ini_get('safe_mode')){if(empty($stimeout)|| $stimeout < (10*60))@set_time_limit(10*60);}
		$stimeout=10*60;
		//set mysql wait timeout to 15 minutes
		$mtimeout=$wpdb->get_var("SELECT @@session.wait_timeout AS mtimeout FROM DUAL");
		if(!empty($mtimeout) && !is_wp_error($mtimeout)){
			if(!is_numeric($mtimeout)|| $mtimeout<$timelimit) $results=$wpdb->query(sprintf("SET wait_timeout = %d",($timelimit+60)));
			elseif($mtimeout >3600)$timelimit=3600; //up to 1 hour
			else $timelimit=$mtimeout;
		}else{
			$results=$wpdb->query(sprintf("SET wait_timeout = %d",($timelimit+60)));
		}
		$defs=array();
		$ints=array();
		$cols="";
		$i=0;
		foreach($table_structure as $col){
		//omit ID field from export
		if($exclude_id && $col->Field=="id" && $col->Extra=="auto_increment")continue;
		//differentiate numeric fields from char fields
		if((0===strpos(strtolower($col->Type),'tinyint'))||
		  (0===strpos(strtolower($col->Type),'smallint'))||
		  (0===strpos(strtolower($col->Type),'mediumint'))||
		  (0===strpos(strtolower($col->Type),'int'))||
		  (0===strpos(strtolower($col->Type),'bigint'))||
		  (0===strpos(strtolower($col->Type),'timestamp'))){
			$defs[$col->Field]=$col->Default;
			$ints[$col->Field]="1";
		}
		if($i==0)$cols .='`'.$col->Field.'`';
		else $cols .=', `'.$col->Field.'`';
		$i++;
		} //end foreach $table_structure
		
		// Batch by $row_inc
		if($segment=='none'){$row_start=0;$row_inc=ROWS_PER_SEGMENT;}
		else{$row_start=$segment*ROWS_PER_SEGMENT;$row_inc=ROWS_PER_SEGMENT; }
		$i=1;
		$entries="INSERT INTO `".esc_attr($table)."` ($cols) VALUES ";
		$search=array("\x00", "\x0a", "\x0d", "\x1a");
		$replace=array('\0','\n','\r','\Z');
		$last_recid=0;
		$export_count=0;
		$err_msg="";
		do{
			$row_count=0;
			$qry=sprintf("SELECT SQL_BUFFER_RESULT $cols FROM %s %s ORDER BY `id` LIMIT %d, %d",esc_attr($table),$condition,$row_start,$row_inc);
			$table_data=$wpdb->get_results($qry);
			if(is_wp_error($table_data)) {
				$errno=$table_data->get_error_code();
				$err_msg=sprintf(__('Error exporting data from table %s: %s','wassup'), esc_attr($table),$errno.' '.$table_data->get_error_message());
			}elseif(is_array($table_data)){
				$row_count=count($table_data);
			}
			if($row_count>0){
				$j=0;
			foreach($table_data AS $row){
				$values="";$row_sql="";$n=0;
				foreach($row AS $key=>$value){
				//track last id exported
				if($key=='id'){
					$last_recid=$value;
					//omit 'id' from export
					if($exclude_id)continue;
				}
				if(isset($ints[$key])){$val=(''===$value)?"''":$value;}
				elseif(function_exists('esc_sql')){$val="'".str_replace($search,$replace,esc_sql($value))."'";}
				else{$val="'".str_replace($search,$replace,addslashes($value))."'";}
				if($n==0)$values=$val;
				else $values .=','.$val;
				$n++;
				} //end foreach row
				//new "insert" statement every 50 rows
				if($j==0){$row_sql="\n".$entries."\n";}
				elseif($j%50==0){$row_sql .=";\n".$entries."\n";}
				else{$row_sql .=",\n";}
				$row_sql .='('.$values.')';
				$sql .=$row_sql;
				$j++;
			} //end foreach table_data
				if(!empty($sql))$sql .=";\n"; //ending semicolon
				$row_start +=$row_count; //$row_start += $row_inc;
				$export_count=$export_count+$row_count;
				$did_export=true;
			} //end if row_count >0
			$i++;
			//stop export if near script timeout limit 
			if((time()-$stimer_start)>$stimeout-10 || !empty($err_msg)){
				$did_export=false;
				break;
			}
		}while($row_count >0 &&($segment=='none'));
		//reset mysql wait timeout to default
		if(empty($mtimeout)|| !is_numeric($mtimeout))$mtimeout=60;
		$wpdb->query("SET wait_timeout=$mtimeout");
		// Create footer/closing comment in SQL-file
		if($segment=='none' || $segment<0){
			$sql .="\n";
			if($did_export){
				$sql .="# ".sprintf(__('End of data contents of table %s','wassup'),$table)."\n";
			}else{
				$sql .="# ".sprintf(__('Interrupted data contents of table %s','wassup'),$table)."\n";
				if(!empty($err_msg)) $sql .="# $errmsg\n";
			}
			$sql .="# --------------------------------------------------------\n";
			if($wdebug_mode){
				$elapsed_time=time()-$stimer_start;
				$sql .="# Timeout length=$timelimit seconds.  Elapsed time=$elapsed_time seconds\n";
				$sql .="# --------------------------------------------------------\n";
			}
			$sql .="#\n# ".sprintf(__("End time: %d","wassup"),date('r',current_time('timestamp')))."					    \n";
			$sql .="# ".sprintf(__("%d out of %d records exported.","wassup"),$export_count,$numrecords)." ".sprintf(__("Last record ID: %d","wassup"),$last_recid)." \n";
			$sql .="\n";
		}
		//save export success message for admin alert notice
		if(!empty($sql)){
			$wassup_user_settings['ualert_message']=sprintf(__("%d out of %d records exported.","wassup"),$export_count,$numrecords)." ".sprintf(__("Last record ID: %d","wassup"),$last_recid);
			update_user_option($current_user->ID,'_wassup_settings',$wassup_user_settings);
			$saved=self::update_wassupmeta($table,'_export_recid-'.$current_user->ID,$last_recid,0);
		}
		return $sql;
	} // end backup_table

} //end class wassupDb
} //end if !class_exists

if(!class_exists('wassupURI')){
/**
 * Static class containing methods to format and clean urls/links for safe output.
 * @since v1.9
 * @author Helene D. <http://helenesit.com>
 */
class wassupURI {
	/** Private constructor for true static class - prevents direct creation of object. */
	private function __construct(){}
	/**
	 * Return a value of true if url argument is a root url and false when url constains a subdirectory path or query parameters.
	 *  - renamed from url_rootcheck() function.
	 */
	static function is_root_url($url){
		$isroot=false;
		if(strpos($url,'.')>0){
			$urlparts=parse_url($url);
			if(!empty($urlparts['host']))$isroot=true;
			if(!empty($urlparts['path'])&& $urlparts['path'] !="/")$isroot=false;
			elseif(!empty($urlparts['query']))$isroot=false;
		}
		return $isroot;
	}
	/**
	 * Return a url with "$blogurl" prepended for sites that have wordpress installed in a separate folder.
	 *  - renamed from wAddSiteurl() function.
	 */
	static function add_siteurl($inputurl){
		if(preg_match('/^https?\:/',$inputurl)===false){
			if(function_exists('get_site_url')){
				$outputurl=get_site_url($inputurl);
			}else{
				$siteurl=rtrim(self::get_sitehome(),"/");
				$wpurl=rtrim(self::get_wphome(),"/");
				if(strcasecmp($siteurl,$wpurl)==0)$outputurl=$inputurl;
				elseif(stristr($inputurl,$siteurl)===FALSE && self::is_root_url($siteurl))$outputurl=$siteurl."/".ltrim($inputurl,"/");
				else $outputurl=$inputurl;
				$outputurl=rawurldecode(html_entity_decode($outputurl)); //dangerous
				$outputurl=self::cleanURL($outputurl);	//safe
			}
			return $outputurl;
		}else{
			return $inputurl;
		}
	}
	/** Return the url and "path" for wordpress site's "home". */
	static function get_sitehome(){
		$sitehome=get_option('siteurl');
		if(is_multisite() && is_network_admin()){
			$sitehome=network_home_url();
		}elseif(empty($sitehome)){
			$sitehome=get_option('home');
		}
		return $sitehome;
	} //end get_homeurl

	//** Return the url and "path" for wordpress admin. */
	static function get_wphome(){
		$wphome=admin_url();
		if(is_multisite() && is_network_admin()){
			$wphome=network_admin_url();
		}elseif(empty($wphome)){
			$wphome=get_option('wpurl');
		}
		return $wphome;
	} //end get_wphome

	/** Return a request url or a text string if url is malware/hack attempt or 404. @since v1.9 */
	static function url_link($urlrequested,$chars=0,$spam=0){
		global $wassup_options;
		$urllink=false;
		if(empty($chars)|| !is_numeric($chars))$chars=(int)$wassup_options->wassup_screen_res/10;
		$request=strtolower($urlrequested);
		if(strlen($request)>60)$tooltip=' title="'.self::disarm_attack($request).'" ';
		else $tooltip="";
		//no link for spam or 404 or wordpress admin
		if(preg_match('/^(\[|\%5B)\d{3}(\]|\%5D)/',$urlrequested)>0 || !empty($spam)) $urllink="<span{$tooltip}>".stringShortener("$urlrequested",round($chars*.9,0))."</span>";
		else $urllink='<a href="'.self::add_siteurl("$urlrequested").'" target="_BLANK">'.stringShortener("$urlrequested",$chars).'</a>';
		return $urllink;
	}
	/** Return an external referrer link or a text string if link is internal, or is spam or 404.  @since v1.9 */
	static function referrer_link($referer,$requesturl,$chars=0,$spam=0){
		global $wassup_options;
		$referrerlink=false;
		if(!empty($referer)){
			$wpurl=self::get_wphome();
			$siteurl=self::get_sitehome();
			$adminurl=admin_url("");
			if(empty($chars)|| !is_numeric($chars))$chars=(int)($wassup_options->wassup_screen_res/10);
			$tooltip="";
			$ref=strtolower($referer);
			if(strlen($ref)>60)$tooltip=' title="'.self::disarm_attack($ref).'" ';
			if($spam==0 && stristr($referer,$adminurl)!=$referer && stristr($ref,$siteurl)!=$referer){
				$favicon_img="";
				$rurl=parse_url($referer);
				if(!empty($rurl['host']) && preg_match('/\.[a-z]{2,4}$/',$rurl['host'])>0) $favicon_img='<img src="http://www.google.com/s2/favicons?domain='.$rurl['host'].'" class="favicon"> ';
				//no link for spam
				if(strpos($ref,'http')===false || strpos($ref,'http')>0) $referrerlink=$favicon_img."<span{$tooltip}>".stringShortener($referer,round($chars*.9,0))."</span>";
				else $referrerlink=$favicon_img.'<a href="'.self::disarm_attack($referer).'" target=_"BLANK"'.$tooltip.'>'.stringShortener($referer,round($chars*.9,0)).'</a>';
			}elseif($spam >0){
				//no link for spam
				$referrerlink="<span{$tooltip}>".stringShortener("$referer",round($chars*.9,0))."</span>";
			}elseif(!empty($requesturl) && $ref == self::add_siteurl($requesturl)){
				//when request and referrer are same, this is disguised direct hit
				$referrerlink='<span>'.__("direct hit","wassup").'</span>';
			}elseif(strpos($ref,'/wp-includes/')>0 || strpos($ref,'/wp-content/')>0){
				$referrerlink='<a href="'.self::disarm_attack($referer).'" target=_"BLANK"'.$tooltip.'>'.stringShortener($referer,round($chars*.9,0)).'</a>';
			}else{
				$referrerlink="<span{$tooltip}>".__("from your site","wassup")."</span>";
			}
		}else{
			$referrerlink='<span>'.__("direct hit","wassup").'</span>';
		}
		return $referrerlink;
	} //end referrer_link

	/** Return a referrer link for search engines. @since v1.9 */
	static function se_link($referer,$chars=0,$spam=0){
		global $wassup_options;
		if(empty($chars)|| !is_numeric($chars))$chars=(int)($wassup_options->wassup_screen_res/10);
		$selink=false;
		if(!empty($referer)){
			$tooltip="";
			$ref=strtolower($referer);
			if(strlen($ref)>60)$tooltip=' title="'.self::disarm_attack($referer).'" ';
			if(empty($spam))$selink='<a href="'.self::disarm_attack($referer).'" target=_"BLANK"'.$tooltip.'><span>'.stringShortener($referer,round($chars*.8,0)).'</span></a>';
			else $selink="<span{$tooltip}>".stringShortener($referer,round($chars*.8,0))."</span>";
		}
		return $selink;
	}

	/** Remove all ascii codes and replace with '---' in url. Can be used before saving to database */
	static function neutralize($url){
		$newurl=wp_kses(preg_replace('/(&#37;|&amp;#37;|%)(?:[01][0-9A-F]|7F)/i','---',$url),array());
		return $newurl;
	}
	/**
	 * Return a url with some chars replaced with safer html-encoded versions and ascii codes removed.
	 *  - for displaying questionable data in requesturl, referrer and user agent.
	 * @since v1.9
	 */
	static function disarm_attack($url=false){
		if(!empty($url)&& !is_numeric($url)){
			$newurl=str_replace(array(' ','!','$','"','&&','\'','(',')','*',',','.','<','=','>','?','\\','^','`','{','|','~'),array('&#32;','&#33;','&#36;','&#34;','&#38;&#38;','&#39;','&#40;','&#41;','&#42;','&#44;','&#46;','&#60;','&#61;','&#62;','&#63;','&#92;','&#94;','&#96;','&#123;','&#124;','&#126;'),htmlentities(stripslashes(html_entity_decode(preg_replace('/(&#37;|&amp;#37;|%)(?:[01][0-9A-F]|7F)/i','---',$url)))));
		}else{
			$newurl=$url;
		}
		return $newurl;
	}
	/** Return a url that is sanitized of all potentially dangerous code. */
	static function cleanURL($url=""){
		if(!empty($url)&& !is_numeric($url)){
			$loc=self::disarm_attack($url);
			$allowed=array('http','https','ftp','ftps','mailto','news','irc','gopher','nntp','feed','telnet','mms','rtsp','svn','tel','fax','xmpp','webcal');
			$loc=wp_kses_bad_protocol($loc,$allowed);
			$cleaned_url=preg_replace(array('/([^a-df-wyz])\-/i','/\-([^a-df-wyz])/i','#\:([^/]{2})#','/([^0-9]{2,3})\;/','/([^&])#/'),array('\1&#45;','&#45;\1','&#58;\1','\1&#59;','\1&#35;'),esc_attr($loc));
		}else{
			$cleaned_url=$url;
		}
		return $cleaned_url;
	} //end cleanURL

	/**
	 * Return the 'page' query parameter or the menu link query parameter 'ml' for a wassup-stats page from the URI
	 * @param none
	 * @return string
	 */
	static function get_menu_arg(){
		$menuarg=false;
		if(isset($_GET['page'])) $menuarg=$_GET['page'];
		if(stristr($menuarg,"wassup")!==false){
			$wassupfolder=basename(WASSUPDIR);
			if($menuarg=="wassup-stats"){
				if(!empty($_GET['ml'])) $menuarg=$_GET['ml'];
				else $menuarg="wassup";
			}elseif($menuarg=="wassup-spia"){
				$menuarg="wassup-spy";
			}elseif($menuarg==$wassupfolder){
				$menuarg="wassup";
			}
		}
		return $menuarg;
	}
	/** check for admin uri with valid referrer via _wpnonce */
	static function is_valid_admin_referer($action="-1",$wpage=""){
		global $wp_version;
		$is_valid_referer=false;
		if(version_compare($wp_version,'2.8','>=') || $action=="-1"){
			$is_valid_referer=check_admin_referer($action);
		}elseif(is_admin() && !empty($_SERVER['HTTP_REFERER'])){
			if(isset($_REQUEST['_wpnonce']) && wp_verify_nonce($_REQUEST['_wpnonce'],$action)){
				if(isset($_REQUEST['_wp_http_referer'])){
					if(strpos($_SERVER['HTTP_REFERER'],$_REQUEST['_wp_http_referer'])!==false) $is_valid_referer=true;
				}else{
					if(empty($wpage)) $wpage=self::get_menu_arg();
					if(!empty($wpage) && strpos($_SERVER['HTTP_REFERER'],"page=$wpage")!==false) $is_valid_referer=true;
				}
			}
		}
		return $is_valid_referer;
	}
} //end Class wassupURI
} //end if !class_exists
?>
